/*
 * This file is part of Geomajas, a component framework for building
 * rich Internet applications (RIA) with sophisticated capabilities for the
 * display, analysis and management of geographic information.
 * It is a building block that allows developers to add maps
 * and other geographic data capabilities to their web applications.
 *
 * Copyright 2008-2010 Geosparc, http://www.geosparc.com, Belgium
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.geomajas.plugin.geocoder.command.dto;

import org.geomajas.command.CommandRequest;
import org.geomajas.global.Api;

/**
 * Request object for getting location information for a string.
 *
 * @author Joachim Van der Auwera
 * @since 1.0.0
 */
@Api(allMethods = true)
public class GetLocationForStringRequest implements CommandRequest {

	private static final long serialVersionUID = 100L;

	public static final int MAX_ALTERNATIVES_UNLIMITED = 0;
	public static final int MAX_ALTERNATIVES_DEFAULT = 50;

	private String location;
	private String crs;
	private String servicePattern = ".*";
	private String locale;
	private int maxAlternatives = MAX_ALTERNATIVES_DEFAULT;

	/**
	 * Get the location string to search for.
	 *
	 * @return location string
	 */
	public String getLocation() {
		return location;
	}

	/**
	 * Set the location string to search for.
	 *
	 * @param location location string
	 */
	public void setLocation(String location) {
		this.location = location;
	}

	/**
	 * Get the coordinate reference space which should be used for the returned location.
	 *
	 * @return The map's coordinate reference system.
	 */
	public String getCrs() {
		return crs;
	}

	/**
	 * Set the coordinate reference space which should be used for the returned location.
	 *
	 * @param crs The map's coordinate reference system.
	 */
	public void setCrs(String crs) {
		this.crs = crs;
	}

	/**
	 * Get the regular expression for the geocoder names which should be used for the search.
	 *
	 * @return regular expression for selecting the geocoders to use
	 */
	public String getServicePattern() {
		return servicePattern;
	}

	/**
	 * Set the regular expression for the geocoder names which should be used for the search.
	 *
	 * @param servicePattern regular expression for selecting the geocoders to use
	 */
	public void setServicePattern(String servicePattern) {
		this.servicePattern = servicePattern;
	}

	/**
	 * Get the locale which should be used for the search, if known.
	 *
	 * @return locale string if known, null if not known
	 */
	public String getLocale() {
		return locale;
	}

	/**
	 * Set the locale which should be used for the search if known.
	 *
	 * @param locale locale string if known, null if not known
	 */
	public void setLocale(String locale) {
		this.locale = locale;
	}

	/**
	 * Get the maximum number of alternatives which may be returned.
	 * <p/>
	 * The default value is to have 50 alternatives returned.
	 *
	 * @return max number of features to return of {@link #MAX_ALTERNATIVES_UNLIMITED} (0) for unlimited.
	 */
	public int getMaxAlternatives() {
		return maxAlternatives;
	}

	/**
	 * Set the maximum number of alternatives which may be returned.
	 *
	 * @param maxAlternatives max number of alternatives to return, or {@link #MAX_ALTERNATIVES_UNLIMITED} (0) for
	 * unlimited (which is not recommended to use).
	 */
	public void setMaxAlternatives(int maxAlternatives) {
		this.maxAlternatives = maxAlternatives;
	}

}
